
#include "actor.h"



// Initialize the static display to NULL.
Display *Actor::display = NULL;
Mixer *Actor::mixer = NULL;



Actor::Actor()
{
	hits_enemies = false;
	hits_players = false;
	x = 0;
	y = 0;
	z = 0;
	type = "Actor";
	anim = NULL;
	behavior = NULL;
	bounding_box.set(0, 0, 0, 0);
	if (display == NULL)
		std::cerr << "WARNING: Actor instantiated before being bound to a display" << std::endl;
	actorlist = NULL;
	dies_on_leaving_scope = false;
	scope.set(0, 0, 0, 0);
}



Actor::~Actor()
{
	if (strcmp(type, "Actor") == 0) {
		if (anim != NULL) delete anim;
		if (behavior != NULL) delete behavior;
	}
}



void Actor::bind_display(Display *new_display)
{
	display = new_display;
}



void Actor::bind_mixer(Mixer *new_mixer)
{
	mixer = new_mixer;
}



void Actor::set_actorlist(ActorList *new_actorlist)
{
	actorlist = new_actorlist;
}



ActorList *Actor::get_actorlist()
{
	return actorlist;
}



void Actor::set_anim(Anim *new_anim)
{
	anim = new_anim;
}



void Actor::set_behavior(Behavior *new_behavior)
{
	behavior = new_behavior;
	behavior->set_position(x, y);
}



void Actor::set_collision_damage(int damage)
{
	collision_damage = damage;
}



int Actor::get_collision_damage()
{
	return collision_damage;
}



char *Actor::get_current_frame_tag()
{
	return anim->get_current_frame();
}



BoundingBox Actor::get_bounding_box()
{
	return bounding_box;
}



Display *Actor::get_display()
{
	return display;
}



Mixer *Actor::get_mixer()
{
	return mixer;
}



// Tell the display to paint the actor.
void Actor::paint()
{
	if (display == NULL) {
		std::cerr << "WARNING: tried to paint an Actor before display bound" << std::endl;
	} else if (anim == NULL) {
		std::cerr << "WARNING: tried to paint an Actor without an anim" << std::endl;
	} else {
		display->paint_sprite(x, y, anim->get_current_frame(), anim->get_hash());
	}
}



Actor::ActorSignal Actor::update()
{
	ActorSignal empty = {"", this};
	update_base();
	if (dies_on_leaving_scope && !scope.contains(bounding_box)) {
		empty.signal = "suicide";
		empty.source = this;
	}
	return empty;
}



Actor::ActorSignal Actor::collided()
{
	ActorSignal signal;
	signal.signal = "hurt";
	signal.source = this;
	return signal;
}



void Actor::get_signal(ActorSignal signal)
{
}



void Actor::set_position(const int new_x, const int new_y)
{
	x = new_x;
	y = new_y;
	if (behavior != NULL)
		behavior->set_position(new_x, new_y);
}



void Actor::set_z(const int new_z)
{
	z = new_z;
}



Behavior *Actor::get_behavior()
{
	return behavior;
}



Anim *Actor::get_anim()
{
	return anim;
}



int Actor::get_x()
{
	return x;
}



int Actor::get_y()
{
	return y;
}



int Actor::get_z()
{
	return z;
}



char *Actor::get_type()
{
	return type;
}



void Actor::update_base()
{
	if (behavior != NULL) behavior->update(x, y);
	if (anim != NULL) {
		anim->update();
	}
}



void Actor::update_bounding_box()
{
	BoundingBox framesize, offsets, position;
	position.set(x, y, x, y);
	framesize = display->get_sprite_dimensions(anim->get_current_frame());
	offsets = anim->get_current_frame_offsets();
	bounding_box = framesize + offsets + position;
}



void Actor::dump_bounding_box(std::ostream &out)
{
	int x1, y1, x2, y2;
	bounding_box.get(x1, y1, x2, y2);
	out << "(" << x1 << "," << y1 << ")-(" << x2 << "," << y2 << ")" << std::endl;
}



bool Actor::check_collision(const Actor &actor)
{
	if (bounding_box.collides_with(actor.bounding_box))
		return true;
	return false;
}



void Actor::set_damage_types(bool players, bool enemies)
{
	hits_enemies = enemies;
	hits_players = players;
}



bool Actor::damages_enemies()
{
	return hits_enemies;
}



bool Actor::damages_players()
{
	return hits_players;
}



void Actor::set_scope(int x1, int y1, int x2, int y2)
{
	scope.set(x1, y1, x2, y2);
}



void Actor::set_death_on_leaving_scope(bool value)
{
	dies_on_leaving_scope = value;
}



bool Actor::get_death_on_leaving_scope()
{
	return dies_on_leaving_scope;
}



BoundingBox Actor::get_scope()
{
	return scope;
}

