
#include "imagecache.h"



// Initialize the first node of the list to NULL.
ImageCache::ImageCache()
{
	first = NULL;
}



// Clear the list before destruction.
ImageCache::~ImageCache()
{
	clear();
}



// Call remove_node for each item on the list.  (It could be more
// efficient to walk it normally instead, but I'm too lazy.)
void ImageCache::clear()
{
	while (first != NULL)
		remove_node(first);
}



// Load an image from a file into the linked list.
void ImageCache::add_image_from_file(char *file, char *new_tag)
{
	node *new_image = new node;
	char *allocated_tag = new char[strlen(new_tag) + 1];

	strcpy(allocated_tag, new_tag);
		
	//new_image->data = SDL_LoadBMP(file);
	new_image->data = IMG_Load(file);
	
	if (new_image->data == NULL) {
		std::cerr << "FATAL: Unable to load bitmap: " << file << std::endl;
		std::cerr << "Did you get the angelwars-media package?" << std::endl;
		exit(0);
	} else {
		// loaded a bitmap successfully
	}
	SDL_SetAlpha(new_image->data, 0, 0);
	SDL_SetColorKey(new_image->data, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(new_image->data->format, 255, 0, 255));
	new_image->tag = allocated_tag;
	new_image->paint_offset_x = 0;
	new_image->paint_offset_y = 0;
	new_image->next = NULL;
	
	add_node(new_image);
}



// Load an image from a file into the linked list.
void ImageCache::add_image_from_file(char *file, char *new_tag, int off_x, int off_y)
{
	node *new_image = new node;
	char *allocated_tag = new char[strlen(new_tag) + 1];

	strcpy(allocated_tag, new_tag);
		
	new_image->data = SDL_LoadBMP(file);
	
	if (new_image->data == NULL) {
		std::cerr << "FATAL: Unable to load bitmap: " << file << std::endl;
		std::cerr << "Did you get the angelwars-images package?" << std::endl;
		exit(0);
	} else {
		// loaded successfully
	}
	SDL_SetAlpha(new_image->data, 0, 0);
	SDL_SetColorKey(new_image->data, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(new_image->data->format, 255, 0, 255));
	new_image->tag = allocated_tag;
	new_image->paint_offset_x = off_x;
	new_image->paint_offset_y = off_y;
	new_image->next = NULL;
	
	add_node(new_image);
}



// Remove an image from the linked list, by tag.
void ImageCache::remove_image(char *dead_tag)
{
	node *dead_node = find_node(dead_tag);
		
	if (dead_node != NULL) {
		remove_node(dead_node);
	}
}



// Return a pointer to the BITMAP of an image in the list, by tag.
SDL_Surface *ImageCache::get_image(char *search_tag, int &off_x, int &off_y, node *hash)
{
	node *search_node = find_node(search_tag, hash);
	
	if (search_node == NULL) {
		std::cerr << "WARNING: Image " << search_tag << " not found in cache; "
			<< "returning NULL instead." << std::endl;
		return NULL;
	} else {
		//std::cerr << "Returning image " << search_tag << " from cache." << std::endl;
		off_x = search_node->paint_offset_x;
		off_y = search_node->paint_offset_y;
		return search_node->data;
	}
}



BoundingBox ImageCache::get_image_dimensions(char *search_tag)
{
	node *search_node = find_node(search_tag);
	BoundingBox dimensions;
	
	if (search_node == NULL) {
		std::cerr << "WARNING: tried to find dimensions of a nonexisting cache image" << std::endl;
		dimensions.set(0, 0, 0, 0);
	} else {
		dimensions.set(0, 0, (int)search_node->data->w, (int)search_node->data->h);
	}
	return dimensions;
}



// Add a node to the list.  (Private.)
void ImageCache::add_node(node *new_node)
{
	node *walk = first;

	if (first == NULL) {
		first = new_node;
	} else {
		while (walk->next != NULL)
			walk = walk->next;
		walk->next = new_node;
	}
}



// Remove a node from the list.  (Private.)
void ImageCache::remove_node(node *dead_node)
{
	node *walk = first;
	
	if (first == dead_node) {
		first = first->next;
		delete walk;
	} else {
		while ((walk->next != dead_node) && (walk->next != NULL))
			walk = walk->next;
		if (walk->next == NULL) {
			std::cerr << "WARNING: tried to remove a node that isn't in the list."
				<< std::endl;
		} else {
			walk->next = walk->next->next;
			SDL_FreeSurface(dead_node->data);
			delete [] dead_node->tag;
			delete dead_node;
		}
	}
}



// Locate a node, by tag.  (Private.)
ImageCache::node *ImageCache::find_node(char *search_tag, node *hash)
{
	ImageCache::node *walk; // = first;
	
	if (hash != NULL) {
		walk = hash;
	} else {
		walk = first;
	}
		
	if (strcmp(walk->tag, search_tag) == 0)
		return walk;
	while (walk != NULL) {
		if (strcmp(walk->tag, search_tag) == 0)
			return walk;
		walk = walk->next;
	}
	return NULL;
}



void ImageCache::load_images_from_list_file(char *filename)
{
	std::ifstream file;
	char image_file[250];
	char image_tag[250];
	int off_x, off_y;
	
	file.open(filename);
	
	if (file) {
		while (!file.eof()) {
			file >> std::ws;
			file >> image_file >> std::ws >> image_tag >> std::ws >> off_x >> std::ws >> off_y >> std::ws;
			add_image_from_file(image_file, image_tag, off_x, off_y);
		}
	}
}



// Dump debug information (currently the contents of the list) to an std::ostream.
void ImageCache::debug_dump(std::ostream &out)
{
	node *walk = first;

	out << "cache:" << std::endl;
	while (walk != NULL) {
		out << walk->tag << std::endl;
		walk = walk->next;
	}
}






